import matplotlib.pyplot as plt
import pandas as pd
import numpy as np
from scipy.integrate import trapz

# Read the CO2 concentration values from the file
co2_ppm_file_path = "co2_ppm_full.txt"
try:
    co2_ppm_values = pd.read_csv(co2_ppm_file_path, header=None, names=['CO2_ppm'])
except FileNotFoundError:
    print(f"Error: File '{co2_ppm_file_path}' not found.")
    exit()
except pd.errors.EmptyDataError:
    print(f"Error: File '{co2_ppm_file_path}' is empty.")
    exit()

# Read the data from the CO2 bending mode file
co2_bend_file_path = "co2_bend.txt"
try:
    data = pd.read_csv(co2_bend_file_path, header=None, names=['Wavenumber', 'AbsorptionIntensity'])
except FileNotFoundError:
    print(f"Error: File '{co2_bend_file_path}' not found.")
    exit()
except pd.errors.EmptyDataError:
    print(f"Error: File '{co2_bend_file_path}' is empty.")
    exit()

# Initialize lists to store results for each CO2 concentration
trans_values = []
temp_surf_values = []

# Fixed temperature
temp = 288

# Loop through each CO2 concentration value
for co2_ppm in co2_ppm_values['CO2_ppm']:
    # Normalize CO2 concentration to ppm
    co2_ppm /= 1e6
    
    # Calculate cross-section number density
    xsec_num_density = 234972 * co2_ppm

    # Multiply each absorption intensity by 96.8 to obtain relative absorbance (Arel)
    data['RelativeAbsorbance'] = data['AbsorptionIntensity'] * xsec_num_density

    # Convert relative absorbance to flux transmittance using the provided formula
    data['FluxTransmittance'] = (np.exp(-(data['RelativeAbsorbance'] * 2.245)) + np.exp(-(data['RelativeAbsorbance'] * 1.13))) / 2

    # Generate wavenumbers from 0.01 to 2399.99 cm^-1
    wvn = np.arange(0.01, 2400, 0.01)

    # Calculate Planck distribution for the fixed temperature
    planck_distribution = (0.000000037211 * wvn**3) / (np.exp(1.435 * wvn / temp) - 1)

    # Multiply flux transmittance by Planck distribution at each wavenumber
    data['Result'] = data['FluxTransmittance'] * np.interp(data['Wavenumber'], wvn, planck_distribution)

    # Integrate the area under the Planck distribution and the product of the Planck curve with flux transmittance
    integral_planck = trapz(planck_distribution, wvn)
    integral_result = trapz(data['Result'], data['Wavenumber'])

    # Calculate transmittance and surface temperature
    ratio = integral_result / integral_planck
    trans = ratio * 0.83
    temp_surf = 255 / trans**0.25

    # Append results to the lists
    trans_values.append(trans)
    temp_surf_values.append(temp_surf)

# Plotting transmittance vs CO2 concentration
plt.figure(figsize=(10, 6))
plt.plot(co2_ppm_values['CO2_ppm'], trans_values, label='Transmittance vs CO2 Concentration')
plt.xlabel('CO2 Concentration (ppm)')
plt.ylabel('Transmittance')
plt.title('Transmittance vs CO2 Concentration')
plt.legend()

# Plotting surface temperature vs CO2 concentration
plt.figure(figsize=(10, 6))
plt.plot(co2_ppm_values['CO2_ppm'], temp_surf_values, label='Surface Temperature vs CO2 Concentration', color='orange')
plt.xlabel('CO2 Concentration (ppm)')
plt.ylabel('Surface Temperature (K)')
plt.title('Surface Temperature vs CO2 Concentration')
plt.legend()

# Save results to CSV files
result_df = pd.DataFrame({'CO2_ppm': co2_ppm_values['CO2_ppm'], 'Transmittance': trans_values, 'Surface_Temperature': temp_surf_values})
result_df.to_csv('co2_ppm_vs_trans.csv', index=False)
print("Saved results to 'co2_ppm_vs_trans.csv'.")

# Show the plots
plt.show()